@@ -223,9 +223,6 @@ int link_find_tclass(Link *link, uint32_t classid, TClass **ret) {
                 if (tclass->classid != classid)
                         continue;
 
-                if (tclass->source == NETWORK_CONFIG_SOURCE_FOREIGN)
-                        continue;
-
                 if (!tclass_exists(tclass))
                         continue;
 
@@ -255,6 +252,58 @@ static void log_tclass_debug(TClass *tclass, Link *link, const char *str) {
                        strna(tclass_get_tca_kind(tclass)));
 }
 
+void tclass_mark_recursive(TClass *tclass) {
+        QDisc *qdisc;
+
+        assert(tclass);
+        assert(tclass->link);
+
+        if (tclass_is_marked(tclass))
+                return;
+
+        tclass_mark(tclass);
+
+        /* Also mark all child qdiscs assigned to the class. */
+        SET_FOREACH(qdisc, tclass->link->qdiscs) {
+                if (qdisc->parent != tclass->classid)
+                        continue;
+
+                qdisc_mark_recursive(qdisc);
+        }
+}
+
+void link_tclass_drop_marked(Link *link) {
+        TClass *tclass;
+
+        assert(link);
+
+        SET_FOREACH(tclass, link->tclasses) {
+                if (!tclass_is_marked(tclass))
+                        continue;
+
+                tclass_unmark(tclass);
+                tclass_enter_removed(tclass);
+
+                if (tclass->state == 0) {
+                        log_tclass_debug(tclass, link, "Forgetting");
+                        tclass_free(tclass);
+                } else
+                        log_tclass_debug(tclass, link, "Removed");
+        }
+}
+
+TClass* tclass_drop(TClass *tclass) {
+        assert(tclass);
+
+        tclass_mark_recursive(tclass);
+
+        /* link_tclass_drop_marked() may invalidate tclass, so run link_qdisc_drop_marked() first. */
+        link_qdisc_drop_marked(tclass->link);
+        link_tclass_drop_marked(tclass->link);
+
+        return NULL;
+}
+
 static int tclass_handler(sd_netlink *rtnl, sd_netlink_message *m, Request *req, Link *link, TClass *tclass) {
         int r;
 
@@ -315,7 +364,7 @@ static bool tclass_is_ready_to_configure(TClass *tclass, Link *link) {
         if (!IN_SET(link->state, LINK_STATE_CONFIGURING, LINK_STATE_CONFIGURED))
                 return false;
 
-        return link_find_qdisc(link, tclass->classid, tclass->parent, tclass_get_tca_kind(tclass), NULL) >= 0;
+        return link_find_qdisc(link, TC_H_MAJ(tclass->classid), tclass_get_tca_kind(tclass), NULL) >= 0;
 }
 
 static int tclass_process_request(Request *req, Link *link, TClass *tclass) {
@@ -464,14 +513,9 @@ int manager_rtnl_process_tclass(sd_netlink *rtnl, sd_netlink_message *message, M
                 break;
 
         case RTM_DELTCLASS:
-                if (tclass) {
-                        tclass_enter_removed(tclass);
-                        if (tclass->state == 0) {
-                                log_tclass_debug(tclass, link, "Forgetting");
-                                tclass_free(tclass);
-                        } else
-                                log_tclass_debug(tclass, link, "Removed");
-                } else
+                if (tclass)
+                        (void) tclass_drop(tclass);
+                else
                         log_tclass_debug(tmp, link, "Kernel removed unknown");
 
                 break;
@@ -483,6 +527,21 @@ int manager_rtnl_process_tclass(sd_netlink *rtnl, sd_netlink_message *message, M
         return 1;
 }
 
+int link_enumerate_tclass(Link *link, uint32_t parent) {
+        _cleanup_(sd_netlink_message_unrefp) sd_netlink_message *req = NULL;
+        int r;
+
+        assert(link);
+        assert(link->manager);
+        assert(link->manager->rtnl);
+
+        r = sd_rtnl_message_new_traffic_control(link->manager->rtnl, &req, RTM_GETTCLASS, link->ifindex, 0, parent);
+        if (r < 0)
+                return r;
+
+        return manager_enumerate_internal(link->manager, link->manager->rtnl, req, manager_rtnl_process_tclass);
+}
+
 static int tclass_section_verify(TClass *tclass) {
         int r;
 
@@ -523,13 +582,12 @@ int config_parse_tclass_parent(
                 void *userdata) {
 
         _cleanup_(tclass_free_or_set_invalidp) TClass *tclass = NULL;
-        Network *network = data;
+        Network *network = ASSERT_PTR(data);
         int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
-        assert(data);
 
         r = tclass_new_static(ltype, network, filename, section_line, &tclass);
         if (r == -ENOMEM)
@@ -570,13 +628,12 @@ int config_parse_tclass_classid(
                 void *userdata) {
 
         _cleanup_(tclass_free_or_set_invalidp) TClass *tclass = NULL;
-        Network *network = data;
+        Network *network = ASSERT_PTR(data);
         int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
-        assert(data);
 
         r = tclass_new_static(ltype, network, filename, section_line, &tclass);
         if (r == -ENOMEM)
